Game = {}
Game.__index = Game

function Game.create()
	local t = {}
	setmetatable(t,Game)
	t.level = 1
	t.worlds = {
		World.create(1, 1, t),
		World.create(1, 2, t)
	}

	t.time = 0
	t.deads = 0

   return t
end
function Game:update(dt)
	self.time = self.time+dt
	local numonend = 0
	for i,v in ipairs(self.worlds) do
		v:update(dt)
		if v.onend then
			numonend = numonend+1
		end
	end
	if numonend >= #self.worlds then
		Sounds.win:play()
		self:loadlevel(self.level+1)
	end
end
function Game:draw()
	for i,v in ipairs(self.worlds) do
		v:draw()
	end
end
function Game:keypressed(k)
	for i,v in ipairs(self.worlds) do
		v:keypressed(k)
	end
	if ConWorlds.debug then
		if k == "+" or k == "kp+" then
			self:loadlevel(self.level+1)

		elseif k == "-" or k == "kp-" then
			self:loadlevel(self.level-1)
		elseif k == "s" and lk.isDown("lctrl") then
			self:savelevel()
		elseif k == "f1" then
			table.insert(Levels, {
				[1] = {
					pos = "top",
					width = 4000,
					height = 2000,
					playerx = 50,
					playery = 1800,
					background = {90, 200, 255},
					elements = {

					}
				},
				[2] = {
					pos = "bottom",
					width = 4000,
					height = 2000,
					playerx = 50,
					playery = 1800,
					background = {255, 155, 112},
					elements = {

					}
				}
			})
			self:loadlevel(#Levels)
		elseif k == "f2" then
			table.insert(Levels, {
				[1] = {
					pos = "topleft",
					width = 4000,
					height = 2000,
					playerx = 50,
					playery = 1800,
					background = {90, 200, 255},
					elements = {

					}
				},
				[2] = {
					pos = "bottomleft",
					width = 4000,
					height = 2000,
					playerx = 50,
					playery = 1800,
					background = {255, 155, 112},
					elements = {

					}
				},
				[3] = {
					pos = "topright",
					width = 4000,
					height = 2000,
					playerx = 50,
					playery = 1800,
					background = {255, 155, 112},
					elements = {

					}
				},
				[4] = {
					pos = "bottomright",
					width = 4000,
					height = 2000,
					playerx = 50,
					playery = 1800,
					background = {90, 200, 255},
					elements = {

					}
				}

			})
			self:loadlevel(#Levels)
		end
	end
end
function Game:mousepressed(x, y, button)
	for i,v in ipairs(self.worlds) do
		v:mousepressed(x, y, button)
	end
end
function Game:mousereleased(x, y, button)
	for i,v in ipairs(self.worlds) do
		v:mousereleased(x, y, button)
	end
end

function Game:reset()
	self.deads = self.deads +1
	for i,v in ipairs(self.worlds) do
		v:reset()
	end
end
function Game:loadlevel(levelid)
	local level = Levels[levelid]
	if type(level) == "table" then
		self.level = levelid
		self.worlds = {}
		for i,v in ipairs(level) do
			table.insert(self.worlds, World.create(levelid, i, self))
		end
		return true
	else
		print("couldn't load level: "..tostring(levelid))
		ConWorlds.time = self.time
		ConWorlds.deads = self.deads
		ConWorlds.ends = EndScreen.create()
		ConWorlds.state = "end"
		return false
	end
end
function Game:savelevel()
	for i,v in ipairs(self.worlds) do
		Levels[self.level][i] = table.copy(v.levelworld, true)
	end
	local content = Tserial.pack(Levels)
	lf.write("slevels.lua", content)
end