Menu = {}
Menu.__index = Menu

local INDENT = 30

function Menu.create()
	local t = {}
	setmetatable(t,Menu)

	t.entries = {}
	if ConWorlds.progress and ConWorlds.progress > 1 then
		table.insert(t.entries, {text="Continue game", indent = 0, id = "continue"})
		table.insert(t.entries, {text="Select level", indent = 0, id = "select"})
	else
		table.insert(t.entries, {text="Start game", indent = 0, id = "start"})
		
	end
	table.insert(t.entries, {text="Quit", indent = 0, id = "quit"})
	lg.setBackgroundColor(90, 200, 255)

	t.selected = 1
	t.lastmousex = lm.getX()
	t.lastmousey = lm.getY()

	return t
end
function Menu:update(dt)
	local changemouse = false
	if self.lastmousex ~= lm.getX() or self.lastmousey ~= lm.getY() then
		changemouse = true
		self.lastmousex = lm.getX()
		self.lastmousey = lm.getY()
	end
	local starty = lg.getHeight()/2 - (#self.entries/2)*font:getHeight()*1.2
	for i,v in ipairs(self.entries) do
		if changemouse and lm.getY()>starty and lm.getY()<starty+font:getHeight()*1.2 then
			self.selected = i
		end

		if i == self.selected then
			v.indent = math.min(v.indent+200*dt, INDENT)
		else
			v.indent = math.max(v.indent-200*dt, 0)
		end
		starty = starty + font:getHeight()*1.2
	end
end
function Menu:draw()
	lg.setColor(255, 255, 255)
	lg.setFont(font)
	local starty = lg.getHeight()/2 - (#self.entries/2)*font:getHeight()*1.2
	for i,v in ipairs(self.entries) do
		lg.setColor(255, 255, 255)
		lg.print(v.text, lg.getWidth()/3+v.indent, starty)
		
		if v.indent > 0 then
			lg.setColor(255, 255, 255, (v.indent/INDENT)*255)
			local x, y = lg.getWidth()/3+v.indent-INDENT, starty
			lg.polygon("fill", x, y+10,    x, y+font:getHeight()-10,    x+20, y+font:getHeight()/2)
		end

		starty = starty + font:getHeight()*1.2
	end
end
function Menu:mousepressed(x, y, button)
	self:keypressed("return")
end
function Menu:keypressed(k)
	if k == "down" then
		self.selected = self.selected + 1
		self.selected = ((self.selected-1) % #self.entries)+1
	elseif k == "up" then
		self.selected = self.selected - 1
		self.selected = self.selected + #self.entries
		self.selected = ((self.selected-1) % #self.entries)+1
	elseif k == "return" or k == " " then
		local selected = self.entries[self.selected].id
		if selected == "start" then
			ConWorlds.game = Game.create()
			ConWorlds.game:loadlevel(1)
			ConWorlds.state = "game"
		elseif selected == "quit" then
			love.event.quit()
		end
	elseif k == "escape" then
		love.event.quit()
	end
	print(self.selected)
end