World = {}
World.__index = World


-- Constants in pixel per second
local SPEEDX = 600
local SPEEDY = 600
local ACCELERATIONX = 1200
local ACCELERATIONY = 1200
local FRICTION = 1200
local JUMP = -600

local types = {
	"platform",
	"spikeup",
	"spikedown",
	"winplate",
	"text"
}

function World.create(levelid, worldid, game)
	local t = {}
	setmetatable(t,World)

	t.game = game
	t.levelworld = table.copy(Levels[levelid][worldid], true)
	t.elements = t.levelworld.elements
	t.player = {
		x = t.levelworld.playerx,
		y = t.levelworld.playery,
		w = 50,
		h = 50,
		vx = 0,
		vy = 0,
		col = {255,216,0}
	}
	t.camera = {
		x = 0,
		y = 0
	}
	t.boxpos = t.levelworld.pos
	t.box = {
		x = 0,
		y = 0,
		w = 100,
		h = 100
	}

	t.onend = false

	t.buildmode = false
	t.buildtype = 1
	t.buildstart = nil

	t.rlearned = false
	t.level = levelid
	return t
end
function World:update( dt )
	dt = math.min(dt, 1/20)
	if self.boxpos == "top" then
		self.box.x = 0
		self.box.y = 0
		self.box.w = lg.getWidth()
		self.box.h = lg.getHeight()/2

	elseif self.boxpos == "bottom" then
		self.box.x = 0
		self.box.y = lg.getHeight()/2
		self.box.w = lg.getWidth()
		self.box.h = lg.getHeight()/2
	elseif self.boxpos == "topleft" then
		self.box.x = 0
		self.box.y = 0
		self.box.w = lg.getWidth()/2
		self.box.h = lg.getHeight()/2
	elseif self.boxpos == "bottomleft" then
		self.box.x = 0
		self.box.y = lg.getHeight()/2
		self.box.w = lg.getWidth()/2
		self.box.h = lg.getHeight()/2
	elseif self.boxpos == "topright" then
		self.box.x = lg.getWidth()/2
		self.box.y = 0
		self.box.w = lg.getWidth()/2
		self.box.h = lg.getHeight()/2
	elseif self.boxpos == "bottomright" then
		self.box.x = lg.getWidth()/2
		self.box.y = lg.getHeight()/2
		self.box.w = lg.getWidth()/2
		self.box.h = lg.getHeight()/2
	else
		error("'"..self.boxpos.."' is not a valid position")
	end

	---- Update Player ----


	if self.buildmode then
		self:updatebuild(dt)
	else

		local move = false
		if lk.isDown("left", "a") then
			self.player.vx = self.player.vx - ACCELERATIONX * dt
			move = true
		end
		if lk.isDown("right", "d") then
			self.player.vx = self.player.vx + ACCELERATIONX * dt
			move = true
		end
		if lk.isDown(" ", "w", "up") then
			if self.player.onground then
				self.player.vy = JUMP
			end
		end
		
		if not move then

			-- Friction
			if self.player.vx > 0 then
				self.player.vx = self.player.vx - FRICTION * dt

				self.player.vx = math.max(self.player.vx, 0)
			elseif self.player.vx < 0 then
				self.player.vx = self.player.vx + FRICTION * dt

				self.player.vx = math.min(self.player.vx, 0)
			end
		end

		self.player.vy = self.player.vy + ACCELERATIONY * dt -- Gravity

		self.player.vx = math.min(math.max(self.player.vx, -SPEEDX), SPEEDX)
		self.player.vy = math.min(math.max(self.player.vy, -SPEEDY), SPEEDY)

		local newx, newy

		newx = self.player.x + self.player.vx*dt
		newy = self.player.y + self.player.vy*dt

		self.oldonend = self.onend
		self.onend = false
		-- Check X collision
		local xcollide = false
		for i,v in ipairs(self.elements) do
			if CheckCollision(
			  v.x, v.y, v.w, v.h,
			  newx, self.player.y, self.player.w, self.player.h
			  ) then



				local todo = Items[v.t].collide(self, v, "x") or {}
					
				
				if todo.returnnow then
					return
				end
				if todo.collide == nil or todo.collide == true then
					xcollide = true
					if self.player.vx > 0 then
						self.player.x = v.x-self.player.w
					else
						self.player.x = v.x+v.w
					end
					if todo.resetvel == nil or todo.resetvel == true then
						self.player.vx = 0
					end
					break
				end


			end
		end
		if not xcollide then 
			self.player.x = newx
		end

		-- Check Y collision
		self.player.onground = false
		local ycollide = false
		for i,v in ipairs(self.elements) do
			if CheckCollision(
			  v.x, v.y, v.w, v.h,
			  self.player.x, newy, self.player.w, self.player.h
			  ) then

				

				local todo = Items[v.t].collide(self, v, "y") or {}

				if todo.returnnow then
					return
				end

				
				if todo.collide == nil or todo.collide == true then
					ycollide = true
					if self.player.vy > 0 then
						self.player.onground = true

						self.player.y = v.y-self.player.h
					else
						self.player.y = v.y+v.h
					end
					if todo.resetvel == nil or todo.resetvel == true then
						self.player.vy = 0
					end
					break
				end
			end
		end
		if not ycollide then 
			self.player.y = newy
		else
			
		end
	end
	-- Update Camera
	self.camera.x = self.player.x - self.box.w/2 + self.player.w/2  -- Center camera to player
	self.camera.y = self.player.y - self.box.h/2 + self.player.h/2  -- Center camera to player

	if self.levelworld.width > self.box.w then
		if self.camera.x < 0 then
			self.camera.x = 0
		elseif self.camera.x > self.levelworld.width - self.box.w then
			self.camera.x = self.levelworld.width - self.box.w
		end
	end
	if self.levelworld.height > self.box.h then
		if self.camera.y < 0 then
			self.camera.y = 0
		elseif self.camera.y > self.levelworld.height - self.box.h then
			self.camera.y = self.levelworld.height - self.box.h
		end
	end
end
function World:draw(  )
	

	lg.setScissor( self.box.x, self.box.y, self.box.w, self.box.h )
	
	-- Draw background
	lg.setColor(self.levelworld.background)
	lg.rectangle("fill", self.box.x, self.box.y, self.box.w, self.box.h)  


	for i,v in ipairs(self.elements) do  -- Draw elements
		if v.col then
			lg.setColor(v.col)
		else
			lg.setColor(255,255,255)
		end
		local sx, sy = self:worldToScreen(v.x, v.y)
		Items[v.t].draw(sx, sy, v.w, v.h, self, v)
	end

	if self.buildmode then
		self:drawbuild()
	end

	-- Draw player
	lg.setColor(self.player.col)
	local psx, psy = self:worldToScreen(self.player.x, self.player.y)
	lg.rectangle("fill", psx, psy, self.player.w, self.player.h)

	lg.setScissor( )
	lg.setColor(0, 0, 0)
	lg.setFont(smallfont)
	-- lg.print("x:"..self.player.x.."\ny:"..self.player.y , self.box.x, self.box.y)
	-- if self.buildstart then
	-- 	lg.print("buildstart: "..self.buildstart.x.." "..self.buildstart.y, self.box.x, self.box.y+30)
	-- end
end

function World:keypressed(k)
	if k == "r" then
		if self.level == 1 and not self.rlearned and not ConWorlds.debug then
			table.insert(self.elements,{t="platform", x = 2300, y=1300, w=150, h=50})
			self.rlearned = true
		end
		self:resetPlayer()
	end
	if ConWorlds.debug then
		if k == "b" then
			self.buildmode = not self.buildmode
		end
		
		if k == "f9" then
			table.insert(self.elements,{t="platform", x = -5, y=0, w=5, h = self.levelworld.height})
			table.insert(self.elements,{t="platform", x = 0, y=-5, w=self.levelworld.width, h = 5})
			table.insert(self.elements,{t="platform", x = self.levelworld.width, y=0, w=5, h = self.levelworld.height})
			table.insert(self.elements,{t="platform", x = 0, y=self.levelworld.height, w=self.levelworld.width, h = 5})
		end
		if k == "c" then
			self.elements = {}
			self.levelworld.elements = self.elements
		end
	end
end
function World:reset()
	self:resetPlayer()
end
function World:resetPlayer()
	self.player.x = self.levelworld.playerx
	self.player.y = self.levelworld.playery
	self.player.vx = 0
	self.player.vy = 0
end
function World:diePlayer()
	Sounds.die:play()
	self.game:reset()
end
function World:mousepressed(x, y, button)
	if self.buildmode then
		if lm.getX() > self.box.x and lm.getX() < self.box.x + self.box.w and
		   lm.getY() > self.box.y and lm.getY() < self.box.y + self.box.h then
			if button == "l" then

				if not self.buildstart and Items[types[self.buildtype]].stype == "drag" then
					local mwx, mwy = self:screenToWorld(lm.getX(), lm.getY())
					local startx = math.floor(mwx/50)*50
					local starty = math.floor(mwy/50)*50
				 	self.buildstart = {x = startx, y = starty, t = types[self.buildtype]}
				else
					local mwx, mwy = self:screenToWorld(lm.getX(), lm.getY())
					local x = math.floor(mwx/50)*50
					local y = math.floor(mwy/50)*50
					Items[types[self.buildtype]].place(self.levelworld, x, y)
				end
			elseif button == "r" then
				local mworldx, mworldy = self:screenToWorld(lm.getX(), lm.getY())
				for i,v in ipairs(self.elements) do
					if mworldx > v.x and mworldx < v.x+v.w and
					   mworldy > v.y and mworldy < v.y+v.h then
					   	table.remove(self.elements, i)
					   	break
					end
				end
			elseif button == "m" then
				local rx, ry = self:screenToWorld(lm.getX(),lm.getY())
				rx, ry = math.floor(rx/50)*50, math.floor(ry/50)*50
				self.levelworld.playerx, self.levelworld.playery = rx, ry
				self:resetPlayer()
			end
		end
		if button == "wd" then
			self.buildtype = self.buildtype+1
			self.buildtype = ((self.buildtype-1)%#types)+1
		elseif button == "wu" then
			self.buildtype = self.buildtype-1+#types
			self.buildtype = ((self.buildtype-1)%#types)+1
		end
	end
end
function World:mousereleased(x, y, button)
	if self.buildmode then
		if self.buildstart then
			local startx = self.buildstart.x
			local starty = self.buildstart.y
			local endx = math.ceil(((lm.getX()-self.box.x)+self.camera.x)/50)*50
			local endy = math.ceil(((lm.getY()-self.box.y)+self.camera.y)/50)*50

			-- Sort, so startx, starty is the top left corner
			if startx >= endx then
				startx, endx = endx-50, startx+50
			end
			if starty >= endy then
				starty, endy = endy-50, starty+50
			end
			if (not (starty == endy)) and (not (startx == endx)) then
				Items[self.buildstart.t].place(self.levelworld, startx, starty, endx, endy)
				print("placed")
			end
			self.buildstart = nil
		end
	end
end

function World:updatebuild(dt)
	if lk.isDown("left", "a") then
		self.player.x = self.player.x - SPEEDX * dt
	end
	if lk.isDown("right", "d") then
		self.player.x = self.player.x + SPEEDX * dt
	end
	if lk.isDown(" ", "w", "up") then
		self.player.y = self.player.y - SPEEDY * dt
	end
	if lk.isDown("lshift", "s", "down") and not lk.isDown("lctrl") then
		self.player.y = self.player.y + SPEEDY * dt
	end

end
function World:drawbuild()
	lg.setColor(150, 150, 150, 100)
	local mwx, mwy = self:screenToWorld(lm.getX(), lm.getY())
	local boxx = math.floor(mwx/50)*50
	local boxy = math.floor(mwy/50)*50
	local x, y = self:worldToScreen(boxx, boxy)

	if lm.getX() > self.box.x and lm.getX() < self.box.x + self.box.w and
   	   lm.getY() > self.box.y and lm.getY() < self.box.y + self.box.h then
		lg.rectangle("fill", x, y, 50, 50)  -- Gridmarker
		lg.setColor(255, 255, 255, 100)

		Items[types[self.buildtype]].drawPreview(x, y)
	end

	if self.buildstart then
		lg.setColor(255, 255, 255)
		local startx = self.buildstart.x
		local starty = self.buildstart.y
		local endx, endy = self:screenToWorld(lm.getX(), lm.getY())
		if startx > endx then
			startx, endx = endx, startx+50
		end
		if starty > endy then
			starty, endy = endy, starty+50
		end
		local screenx, screeny = self:worldToScreen(startx, starty)
		lg.rectangle("fill", screenx, screeny, endx-startx, endy-starty)
	end

end

function World:worldToScreen(wx, wy)
	local sx = wx - self.camera.x + self.box.x
	local sy = wy - self.camera.y + self.box.y

	return sx, sy
end
function World:screenToWorld(sx, sy)
	local wx = sx - self.box.x + self.camera.x
	local wy = sy - self.box.y + self.camera.y

	return wx, wy
end